@@ -1,3 +1,4 @@
+import warnings
 from collections import OrderedDict
 from collections.abc import Mapping, Sequence
 from functools import partial
@@ -10,7 +11,7 @@ from multidict import MultiDict, MultiDictProxy
 
 from .quoting import quote, unquote
 
-__version__ = '0.12.0'
+__version__ = '0.14.2'
 
 __all__ = ['URL']
 
@@ -46,7 +47,7 @@ class cached_property:
         self.wrapped = wrapped
         try:
             self.__doc__ = wrapped.__doc__
-        except:  # pragma: no cover
+        except AttributeError:  # pragma: no cover
             self.__doc__ = ""
         self.name = wrapped.__name__
 
@@ -134,13 +135,14 @@ class URL:
     #               / path-noscheme
     #               / path-empty
     # absolute-URI  = scheme ":" hier-part [ "?" query ]
-    __slots__ = ('_cache', '_val', '_strict')
+    __slots__ = ('_cache', '_val')
 
-    def __init__(self, val='', *, encoded=False, strict=False):
+    def __init__(self, val='', *, encoded=False, strict=None):
+        if strict is not None:  # pragma: no cover
+            warnings.warn("strict parameter is ignored")
         if isinstance(val, URL):
             self._val = val._val
             self._cache = val._cache
-            self._strict = val._strict
             return
         if isinstance(val, str):
             val = urlsplit(val)
@@ -150,8 +152,6 @@ class URL:
         else:
             raise TypeError("Constructor parameter should be str")
 
-        self._strict = strict
-
         if not encoded:
             if not val[1]:  # netloc
                 netloc = ''
@@ -167,26 +167,29 @@ class URL:
                 else:
                     try:
                         ip = ip_address(netloc)
-                    except:
+                    except ValueError:
                         pass
                     else:
                         if ip.version == 6:
-                            netloc = '['+netloc+']'
+                            netloc = '[' + netloc + ']'
                 if val.port:
                     netloc += ':{}'.format(val.port)
                 if val.username:
                     user = _quote(val.username)
-                    if val.password:
-                        user += ':' + _quote(val.password)
+                else:
+                    user = ''
+                if val.password:
+                    user += ':' + _quote(val.password)
+                if user:
                     netloc = user + '@' + netloc
 
-            path = _quote(val[2], safe='+@:', protected='/+', strict=strict)
+            path = _quote(val[2], safe='+@:', protected='/+')
             if netloc:
                 path = _normalize_path(path)
 
             query = _quote(val[3], safe='=+&?/:@',
-                           protected=PROTECT_CHARS, qs=True, strict=strict)
-            fragment = _quote(val[4], safe='?/:@', strict=strict)
+                           protected=PROTECT_CHARS, qs=True)
+            fragment = _quote(val[4], safe='?/:@')
             val = SplitResult(val[0], netloc, path, query, fragment)
 
         self._val = val
@@ -194,7 +197,7 @@ class URL:
 
     @classmethod
     def build(cls, *, scheme='', user='', password='', host='', port=None,
-              path='', query=None, query_string='', fragment='', strict=False):
+              path='', query=None, query_string='', fragment=''):
         """Creates and returns a new URL"""
 
         if host and not scheme:
@@ -220,7 +223,6 @@ class URL:
                 _quote(query_string),
                 fragment
             ),
-            strict=strict,
             encoded=True
         )
 
@@ -282,7 +284,7 @@ class URL:
         return self._val > other._val
 
     def __truediv__(self, name):
-        name = _quote(name, safe=':@', protected='/', strict=self._strict)
+        name = _quote(name, safe=':@', protected='/')
         if name.startswith('/'):
             raise ValueError("Appending path "
                              "starting from slash is forbidden")
@@ -301,15 +303,14 @@ class URL:
                    encoded=True)
 
     def __getstate__(self):
-        return self._val, self._strict
+        return self._val,
 
     def __setstate__(self, state):
         if state[0] is None and isinstance(state[1], dict):
             # default style pickle
             self._val = state[1]['_val']
-            self._strict = state[1]['_strict']
         else:
-            self._val, self._strict = state
+            self._val, *unused = state
         self._cache = {}
 
     def is_absolute(self):
@@ -380,7 +381,10 @@ class URL:
 
         """
         # not .username
-        return self._val.username
+        ret = self._val.username
+        if not ret:
+            return None
+        return ret
 
     @cached_property
     def user(self):
@@ -590,7 +594,7 @@ class URL:
             ret = ret + ':' + str(port)
         if password:
             if not user:
-                raise ValueError("Non-empty password requires non-empty user")
+                user = ''
             user = user + ':' + password
         if user:
             ret = user + '@' + ret
@@ -678,11 +682,11 @@ class URL:
             raise ValueError("host removing is not allowed")
         try:
             ip = ip_address(host)
-        except:
+        except ValueError:
             host = host.encode('idna').decode('ascii')
         else:
             if ip.version == 6:
-                host = '['+host+']'
+                host = '[' + host + ']'
         val = self._val
         return URL(
             self._val._replace(netloc=self._make_netloc(val.username,
@@ -715,7 +719,7 @@ class URL:
     def with_path(self, path, *, encoded=False):
         """Return a new URL with path replaced."""
         if not encoded:
-            path = _quote(path, safe='@:', protected='/', strict=self._strict)
+            path = _quote(path, safe='@:', protected='/')
             if self.is_absolute():
                 path = _normalize_path(path)
         if len(path) > 0 and path[0] != '/':
@@ -723,19 +727,7 @@ class URL:
         return URL(self._val._replace(path=path, query='', fragment=''),
                    encoded=True)
 
-    def with_query(self, *args, **kwargs):
-        """Return a new URL with query part replaced.
-
-        Accepts any Mapping (e.g. dict, multidict.MultiDict instances)
-        or str, autoencode the argument if needed.
-
-        It also can take an arbitrary number of keyword arguments.
-
-        Clear query if None is passed.
-
-        """
-        # N.B. doesn't cleanup query/fragment
-
+    def _get_str_query(self, *args, **kwargs):
         if kwargs:
             if len(args) > 0:
                 raise ValueError("Either kwargs or single query parameter "
@@ -750,7 +742,7 @@ class URL:
         if query is None:
             query = ''
         elif isinstance(query, Mapping):
-            quoter = partial(_quote, qs=True, strict=self._strict)
+            quoter = partial(_quote, qs=True)
             lst = []
             for k, v in query.items():
                 if isinstance(v, str):
@@ -760,56 +752,59 @@ class URL:
                 else:
                     raise TypeError("Invalid variable type: mapping value "
                                     "should be str or int, got {!r}".format(v))
-                lst.append(quoter(k, safe='/?:@')+'='+quoter(v, safe='/?:@;'))
+                lst.append(
+                    quoter(k, safe='/?:@') + '=' + quoter(v, safe='/?:@'))
                 query = '&'.join(lst)
         elif isinstance(query, str):
             query = _quote(query, safe='/?:@',
                            protected=PROTECT_CHARS,
-                           qs=True, strict=self._strict)
+                           qs=True)
         elif isinstance(query, (bytes, bytearray, memoryview)):
             raise TypeError("Invalid query type: bytes, bytearray and "
                             "memoryview are forbidden")
         elif isinstance(query, Sequence):
-            quoter = partial(_quote, qs=True, strict=self._strict)
-            query = '&'.join(quoter(k, safe='/?:@')+'='+quoter(v, safe='/?:@;')
+            quoter = partial(_quote, qs=True, safe='/?:@')
+            query = '&'.join(quoter(k) + '=' + quoter(v)
                              for k, v in query)
         else:
             raise TypeError("Invalid query type: only str, mapping or "
                             "sequence of (str, str) pairs is allowed")
-        path = self._val.path
-        if path == '':
-            path = '/'
+
+        return query
+
+    def with_query(self, *args, **kwargs):
+        """Return a new URL with query part replaced.
+
+        Accepts any Mapping (e.g. dict, multidict.MultiDict instances)
+        or str, autoencode the argument if needed.
+
+        It also can take an arbitrary number of keyword arguments.
+
+        Clear query if None is passed.
+
+        """
+        # N.B. doesn't cleanup query/fragment
+
+        new_query = self._get_str_query(*args, **kwargs)
         return URL(
-            self._val._replace(path=path, query=query), encoded=True)
+            self._val._replace(path=self._val.path, query=new_query),
+            encoded=True)
 
     def update_query(self, *args, **kwargs):
         """Return a new URL with query part updated."""
-        if kwargs:
-            if len(args) > 0:
-                raise ValueError("Either kwargs or single query parameter "
-                                 "must be present")
-            new_query = kwargs
-        elif len(args) == 1:
-            new_query = args[0]
-
-            if isinstance(new_query, str):
-                new_query = OrderedDict(
-                    map(
-                        lambda x: x.split('=', 1),
-                        _quote(new_query,
-                               safe='/?:@', protected=PROTECT_CHARS,
-                               qs=True,
-                               strict=self._strict).lstrip("?").split("&")
-                        )
-                )
-
-        else:
-            raise ValueError("Either kwargs or single query parameter "
-                             "must be present")
-
+        new_query = OrderedDict(
+            map(
+                lambda x: x.split('=', 1),
+                _quote(self._get_str_query(*args, **kwargs),
+                       safe='/?:@', protected=PROTECT_CHARS,
+                       qs=True).lstrip("?").split("&")
+            )
+        )
         query = OrderedDict(self.query)
         query.update(new_query)
-        return self.with_query(tuple(query.items()))
+        query = self._get_str_query(query)
+        return URL(
+            self._val._replace(path=self._val.path, query=query), encoded=True)
 
     def with_fragment(self, fragment):
         """Return a new URL with fragment replaced.
@@ -826,7 +821,7 @@ class URL:
             raise TypeError("Invalid fragment type")
         return URL(
             self._val._replace(
-                fragment=_quote(fragment, safe='?/:@', strict=self._strict)),
+                fragment=_quote(fragment, safe='?/:@')),
             encoded=True)
 
     def with_name(self, name):
